/* interface.c
 * AbsVolume was programmed by Paul Sherman, derived in near 
 * entirety from the program "volumecontrol" by Rob Eberts
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License v2.
 * source code is available at <http://www.pcbypaul.com>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "interface.h"

#include <linux/soundcard.h>
#include <fcntl.h>


typedef struct stereovolume
{
  unsigned char left;
  unsigned char right;
} StereoVolume;
StereoVolume tmpvol; 

struct originalset orig;
unsigned char tmpvolold=-1;
extern int mixer_fd;
GtkWidget *vscale1;
GtkAdjustment *vol_adjustment;
GtkWidget *alignment1;
extern GtkStatusIcon *tray_icon;
int oldicon=-1;

GtkWidget* create_window1 ( GtkStatusIcon *in){
GtkWidget *window1;
  tray_icon=in;
  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (window1, 24, 120);
  gtk_container_set_border_width (GTK_CONTAINER (window1), 1);
  gtk_window_set_resizable (GTK_WINDOW (window1), FALSE);
  gtk_window_set_position (GTK_WINDOW (window1),GTK_WIN_POS_MOUSE);
  gtk_window_set_decorated (GTK_WINDOW (window1), FALSE);
  
  alignment1 = gtk_alignment_new(0.5,0,1,0.65);
  gtk_widget_show(alignment1);
  gtk_container_add (GTK_CONTAINER (window1), alignment1);
  
  vol_adjustment=GTK_ADJUSTMENT (gtk_adjustment_new (1, 0, 99, 1, 10, 0));
  get_current_levels();
  vscale1 = gtk_vscale_new (GTK_ADJUSTMENT (vol_adjustment));
  gtk_widget_show (vscale1);
  gtk_range_set_inverted (GTK_RANGE (vscale1), TRUE);
  gtk_container_add  (alignment1, vscale1);
  gtk_scale_set_draw_value (GTK_SCALE (vscale1), FALSE);

  g_signal_connect ((gpointer) vscale1, "button_release_event", G_CALLBACK (on_vscale1_button_release_event),NULL);
  g_signal_connect ((gpointer) vscale1, "value_changed", G_CALLBACK (on_vscale1_button_release_event),NULL);
  g_object_set_data_full(G_OBJECT (window1), "vscale1",gtk_widget_ref (vscale1),(GDestroyNotify) gtk_widget_unref);
  g_object_set_data (G_OBJECT (window1), "window1", window1);
  g_object_set_data_full (G_OBJECT (window1), "alignment1",gtk_widget_ref (alignment1), (GDestroyNotify) gtk_widget_unref);
  return window1;
}

void get_current_levels() {
	mixer_fd = open ("/dev/mixer", O_RDWR, 0);
 if (mixer_fd < 0){
   printf ("Error opening mixer device\n");
   return (1);
 }
  ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &tmpvol);
  orig.mainvol=tmpvol.left;
  gtk_adjustment_set_value(GTK_ADJUSTMENT(vol_adjustment), (double) tmpvol.left);
  close(mixer_fd);
}

gboolean on_vscale1_button_release_event(GtkWidget *widget,GdkEventButton *event,gpointer user_data){
	mixer_fd = open ("/dev/mixer", O_RDWR, 0);
 if (mixer_fd < 0){
   printf ("Error opening mixer device\n");
   return (1);
 }
  tmpvol.left=tmpvol.right = (double) gtk_range_get_value(GTK_RANGE(vscale1));
  ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_VOLUME), &tmpvol);
 close(mixer_fd);
  timed_func(NULL);
  return FALSE;
}                          	

gboolean timed_func(gpointer data){
	int newicon=-1;
	mixer_fd = open ("/dev/mixer", O_RDWR, 0);
 if (mixer_fd < 0){
   printf ("Error opening mixer device\n");
   return (1);
 }
  ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &tmpvol);
  close(mixer_fd);
  if (((tmpvol.left-tmpvolold)*(tmpvol.left-tmpvolold)<5) && tmpvol.left>5 && tmpvol.left<95){
  	return TRUE;
  }
  tmpvolold=tmpvol.left;
  GdkPixbuf *icon;
  char volchars[3];
  char output[15] = "Volume ";
  sprintf(volchars, "%d", tmpvol.left);
  strcat(output, volchars);
  gtk_status_icon_set_tooltip(tray_icon,&output);
  if (tmpvol.left > 79){
  	newicon=3;
  } else if (tmpvol.left > 29){
  	newicon=2;
  } else if (tmpvol.left > 0){
  	newicon=1;
  }else {
  	newicon=0;
  } 
  if (newicon!=oldicon){
  	if (newicon==3){
    icon = gdk_pixbuf_new_from_file("/usr/local/lib/X11/mini-icons/audio-volume-high.png", NULL);
    if (icon==NULL){
    	fprintf(stderr,"absvolume /usr/local/lib/X11/mini-icons/audio-volume-high.png not found\n");
    }
  	}else if (newicon==2){
    icon = gdk_pixbuf_new_from_file("/usr/local/lib/X11/mini-icons/audio-volume-medium.png", NULL);
    if (icon==NULL){
    	fprintf(stderr,"absvolume /usr/local/lib/X11/mini-icons/audio-volume-medium.png not found\n");
    }
  	}else if (newicon==1){
    icon = gdk_pixbuf_new_from_file("/usr/local/lib/X11/mini-icons/audio-volume-low.png", NULL);
    if (icon==NULL){
    	fprintf(stderr,"absvolume /usr/local/lib/X11/mini-icons/audio-volume-low.png not found\n");
    }
  	}else{
  		icon = gdk_pixbuf_new_from_file("/usr/local/lib/X11/mini-icons/audio-volume-muted.png", NULL);
    if (icon==NULL){
    	fprintf(stderr,"absvolume /usr/local/lib/X11/mini-icons/audio-volume-muted.png not found\n");
    } 
    }
  gtk_status_icon_set_from_pixbuf(tray_icon, icon);
  oldicon=newicon;
  	}
  return TRUE;
}
